from django.contrib import admin, messages
from github.models import Application, User, Authentication, Repository, Key
from github.tasks import fetch_repo


class UserAdmin(admin.ModelAdmin):
    readonly_fields = User._meta.get_all_field_names()
    has_add_permission = lambda s, r: False
    has_delete_permission = lambda s, r, o = None: False

    actions = ['update_repos', 'update', 'update_keys']
    list_display = ('login', 'uid', 'email')

    def update_keys(self, request, queryset):
        for user in queryset.all():
            user.update_keys()

    def update_repos(self, request, queryset):
        for user in queryset.all():
            user.update_repositories()

    def update(self, request, queryset):
        for user in queryset.all():
            user.update_user()


class RepositoryAdmin(admin.ModelAdmin):
    readonly_fields = Repository._meta.get_all_field_names()
    has_add_permission = lambda s, r: False
    has_delete_permission = lambda s, r, o = None: False

    list_display = ('name',
                    'owner',
                    'forks',
                    'watchers',
                    'open_issues',
                    'size',
                    'language',
                    'homepage',
                    'created_at',
                    'updated_at')
    actions = ['refresh', 'fetch_repo']

    date_hierarchy = 'created_at'

    def fetch_repo(self, request, queryset):
        for repository in queryset.all():
            fetch_repo.delay(repository.pk)
            messages.info(request, "Fetching %s" % repository)

    def refresh(self, request, queryset):
        for repository in queryset.all():
            repository.refresh()


admin.site.register(Application,
                    list_display=('name',
                                  'client_id',
                                  'client_secret'))
admin.site.register(User, UserAdmin)
admin.site.register(Authentication,
                    readonly_fields=Authentication._meta.get_all_field_names(),
                    has_add_permission=lambda s, r: False,
                    has_delete_permission=lambda s, r, o=None: False,
                    list_display=('application',
                                  'redirect_uri',
                                  'scope'))

admin.site.register(Repository, RepositoryAdmin)
admin.site.register(Key, list_display=('title', 'user'),
                    readonly_fields=Key._meta.get_all_field_names(),
                    has_add_permission=lambda s, r: False,
                    has_delete_permission=lambda s, r, o=None: False)


import json
import urllib
from django.conf import settings
from django.db import models
import operator
import requests

# 1.4 Compatibility
from github.exceptions import BadVerificationCode

AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


class Application(models.Model):
    name = models.CharField(max_length=100)
    client_id = models.CharField(max_length=20)
    client_secret = models.CharField(max_length=40)
    callback_url = models.URLField(help_text='Needed for auth.')
    default_scope = models.CharField(max_length=100, null=True, blank=True)

    def request(self, path, data=None, params=None, headers=None):
        base_url = 'https://api.github.com/'
        method = "get" if not data else "post"
        headers = dict(headers or {}, Accept='application/json')
        return requests.request(url="%s%s" % (base_url, path),
                                method=method,
                                data=json.dumps(data) if data else None,
                                params=params,
                                headers=headers).json()

    def __unicode__(self):
        return self.name


class Authentication(models.Model):
    application = models.ForeignKey(Application)
    redirect_uri = models.URLField(db_index=True)
    scope = models.CharField(max_length=100, null=True, blank=True)

    def get_absolute_url(self):
        base_url = 'https://github.com/login/oauth/authorize'
        auth_args = {
            'client_id': self.application.client_id,
            'redirect_uri': self.redirect_uri
        }
        if self.scope:
            auth_args['scope'] = self.scope
        elif self.application.default_scope:
            auth_args['scope'] = self.application.default_scope

        auth_args['state'] = self.pk

        return '%s?%s' % (base_url, urllib.urlencode(auth_args))

    def get_access_token(self, code):
        api = 'https://github.com/login/oauth/access_token'
        response = requests.post(api, {
            'client_id': self.application.client_id,
            'redirect_uri': self.redirect_uri,
            'client_secret': self.application.client_secret,
            'code': code
        }, headers={'Accept': 'application/json'}).json()

        if response.get('error') == 'bad_verification_code':
            raise BadVerificationCode()

        return response['access_token']


class User(models.Model):
    user = models.ForeignKey(AUTH_USER_MODEL,
                             null=True,
                             blank=True,
                             related_name='github_users')
    login = models.CharField(max_length=100)
    uid = models.BigIntegerField()
    application = models.ForeignKey(Application)
    avatar_url = models.URLField()
    url = models.URLField()
    email = models.EmailField()

    access_token = models.CharField(max_length=200)

    def request(self, path, data=None, params=None, headers=None):
        params = dict(params or {}, access_token=self.access_token)
        return self.application.request(path, data, params, headers)

    def add_key(self, title, key):
        response = self.request('user/keys', {'title': title, 'key': key})
        Key(user=self).update_from_response(response)

    def update_keys(self):
        response = self.request('user/keys')
        self.keys.exclude(id__in=map(operator.itemgetter('id'),
                                     response)).delete()
        for key_data in response:
            Key(user=self).update_from_response(key_data)

    def update_user(self):
        response = self.request('user')
        self.email = response['email']
        self.url = response['url']
        self.login = response['login']
        self.save()

    def update_repositories(self):
        response = self.request('user/repos')
        self.repositories.all().delete()
        for repository in response:
            assert self.uid == repository['owner']['id']
            obj = Repository(owner=self)
            obj.update_from_response(repository)

    class Meta:
        unique_together = ('uid', 'application')

    def __unicode__(self):
        return self.login


class Key(models.Model):
    user = models.ForeignKey(User, related_name='keys')
    id = models.PositiveIntegerField(primary_key=True)
    key = models.TextField()
    url = models.URLField()
    title = models.CharField(max_length=100)

    def update_from_response(self, key_data):
        self.__dict__.update(key_data)
        self.save()

    def __unicode__(self):
        return self.title


class Repository(models.Model):
    uid = models.BigIntegerField()
    owner = models.ForeignKey(User, related_name='repositories')
    name = models.CharField(max_length=100)
    description = models.TextField()

    private = models.BooleanField()
    fork = models.BooleanField()

    url = models.URLField()
    html_url = models.URLField()
    clone_url = models.URLField()
    git_url = models.CharField(max_length=200)
    ssh_url = models.CharField(max_length=200)
    svn_url = models.URLField()
    mirror_url = models.CharField(max_length=200, null=True, blank=True)

    homepage = models.URLField(null=True, blank=True)
    language = models.CharField(max_length=100, null=True, blank=True)

    forks = models.PositiveSmallIntegerField(default=0)
    forks_count = models.PositiveSmallIntegerField(default=0)
    watchers = models.PositiveSmallIntegerField(default=0)
    watchers_count = models.PositiveSmallIntegerField(default=0)
    size = models.PositiveSmallIntegerField(default=0)

    master_branch = models.CharField(max_length=100, blank=True, null=True)
    open_issues = models.PositiveSmallIntegerField(default=0)

    pushed_at = models.DateTimeField()
    created_at = models.DateTimeField()
    updated_at = models.DateTimeField()

    def refresh(self):
        """ Refresh repo details from Github """
        response = self.owner.request('repos/%s/%s' % (self.owner.login,
                                                       self.name))
        self.update_from_response(response)

    def update_from_response(self, response):
        response['uid'] = response['id']
        del response['id']
        del response['owner']
        self.__dict__.update(response)
        self.save()

    class Meta:
        unique_together = ('uid', 'owner')

    def __unicode__(self):
        return self.name


from django.core.exceptions import ValidationError
from django.core.validators import URLValidator
from django.views.generic import RedirectView
from django.views.generic.detail import SingleObjectMixin
from models import Application, Authentication


class AuthorizeView(RedirectView, SingleObjectMixin):
    model = Application
    permanent = False
    validator = URLValidator()

    def get_redirect_url(self, **kwargs):
        application = self.get_object()

        next = self.request.REQUEST.get('next', application.callback_url)
        try:
            self.validator(next)
        except ValidationError:
            next = self.request.build_absolute_uri(next)

        authentication = Authentication.objects.create(
            application=application,
            redirect_uri=next,
        )
        return authentication.get_absolute_url()



import factory

from .models import Application, Authentication, User


class ApplicationFactory(factory.DjangoModelFactory):
    FACTORY_FOR = Application

    name = 'test'
    client_id = '2134'
    client_secret = 'safsdfdsf'
    callback_url = 'http://testserver'


class AuthenticationFactory(factory.DjangoModelFactory):
    FACTORY_FOR = Authentication

    application = factory.SubFactory(ApplicationFactory)
    redirect_uri = 'http://testserver'
    scope = ''


class UserFactory(factory.DjangoModelFactory):
    FACTORY_FOR = User

    login = 'test'
    uid = 1
    application = factory.SubFactory(ApplicationFactory)
    avatar_url = 'http://example.org/foo.png'
    url = 'http://example.org/foo'
    email = 'foo@example.org'



import os

SITE_ID = 1

APP_ROOT = os.path.abspath(os.path.dirname(__file__))


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

AUTHENTICATION_BACKENDS = (
    'github.backends.GithubBackend',
)

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.admindocs',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django.contrib.sitemaps',
    'django.contrib.sites',

    'github',

    'github.tests.account'
]


MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'github.middlewares.GithubAuthorizationMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

SECRET_KEY = 'foobar'

TEST_RUNNER = 'discover_runner.DiscoverRunner'

TEMPLATE_DIRS = (os.path.join(APP_ROOT, 'templates'),)

ROOT_URLCONF = 'github.tests.urls'

AUTH_USER_MODEL = os.environ.get('AUTH_USER_MODEL', 'auth.User')



# Support for Django 1.4 User models.
def get_user_model():
    try:
        from django.contrib.auth import get_user_model
        return get_user_model()
    except ImportError:
        from django.contrib.auth.models import User
        return User



import json
import urllib
from django.core.urlresolvers import reverse
from django.test import TestCase
from httmock import HTTMock
from mock import patch
from github.exceptions import BadVerificationCode
from github.factories import ApplicationFactory, AuthenticationFactory, UserFactory
from github.models import Authentication, Application, User as GithubUser
from github.utils import get_user_model

User = get_user_model()


class TestAuthorize(TestCase):
    def setUp(self):
        self.app = ApplicationFactory()
        self.url = reverse('github:authorize', kwargs={'pk': self.app.pk})

    def test_redirects_to_github(self):
        response = self.client.get(self.url)
        params = urllib.urlencode({
            'state': '1',
            'redirect_uri': 'http://testserver',
            'client_id': '2134'
        })
        target_url = 'https://github.com/login/oauth/authorize?%s' % params
        # We can replace this in Django 1.7 with
        # self.assertRedirects(.., fetch_redirect_response=False)
        self.assertEqual(response['Location'], target_url)
        self.assertEqual(response.status_code, 302)

    def test_redirect_to_next(self):
        response = self.client.get(self.url, {'next': '/test-url'})
        params = urllib.urlencode({
            'state': '1',
            'redirect_uri': 'http://testserver/test-url',
            'client_id': '2134'
        })
        target_url = 'https://github.com/login/oauth/authorize?%s' % params
        # We can replace this in Django 1.7 with
        # self.assertRedirects(.., fetch_redirect_response=False)
        self.assertEqual(response['Location'], target_url)
        self.assertEqual(response.status_code, 302)


class TestGithubAuthorizationMiddleware(TestCase):
    def setUp(self):
        self.authorization = AuthenticationFactory()
        self.url = "?code=foo&state=%d" % self.authorization.pk

    @patch.object(Authentication, 'get_access_token', lambda s, a: 'foo')
    @patch.object(Application, 'request', lambda s, m, headers: {
        'id': '10',
        'login': 'testuser',
        'url': 'http://example.org/foo',
        'email': 'test@example.org'
    })
    def test_authorize_created_user(self):
        user_count = User.objects.count()
        self.client.get(self.url)
        self.assertEqual(user_count + 1, User.objects.count())

    @patch.object(Authentication, 'get_access_token', lambda s, a: 'foo')
    @patch.object(Application, 'request', lambda s, m, headers: {
        'id': '1',
        'login': 'testuser',
        'url': 'http://example.org/foo',
        'email': 'test@example.org'
    })
    def test_authorize_updates_access_token(self):
        user = UserFactory(application=self.authorization.application)
        self.assertNotEqual(user.access_token, 'foo')
        self.client.get(self.url)
        self.assertEqual(GithubUser.objects.get(pk=user.pk).access_token, 'foo')


class TestAuthorizeModel(TestCase):
    def setUp(self):
        self.authentication = AuthenticationFactory()

    def test_raise_exception_on_invalid_access_token(self):
        response = json.dumps({'error': 'bad_verification_code'})
        with self.assertRaises(BadVerificationCode):
            with HTTMock(lambda url, request: response):
                self.authentication.get_access_token('derp')



import os
from setuptools import setup, find_packages
import github as app

install_requires = [
    'django',
    'django-celery',
    'requests>1.0.0'
]

tests_requires = [
    'django-discover-runner',
    'factory-boy',
    'mock',
    'httmock'
]


def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__), fname)).read()
    except IOError:
        return ''

setup(
    name="django-octocat",
    version=app.__version__,
    description=read('DESCRIPTION'),
    long_description=read('README.rst'),
    license='The MIT License',
    platforms=['OS Independent'],
    keywords='django, app, reusable, github, git',
    author='Kit Sunde',
    author_email='kitsunde.com',
    url="https://github.com/Celc/django-octocat",
    packages=find_packages(),
    include_package_data=True,
    install_requires=install_requires,
    extras_require={
        'tests': tests_requires
    },
)



from django.contrib.auth import authenticate, login
from django.core.urlresolvers import reverse
from django.shortcuts import redirect
from github.exceptions import BadVerificationCode
from github.models import Authentication, User


class GithubAuthorizationMiddleware(object):
    def process_request(self, request):
        if request.GET.get('code') and request.GET.get('state'):
            auth = Authentication.objects.get(pk=request.GET.get('state'))
            try:
                access_token = auth.get_access_token(request.GET['code'])
            except BadVerificationCode:
                return redirect(reverse('github:authorize', kwargs={
                    'pk': auth.application.pk
                }))

            user = auth.application.request('user', headers={
                'Authorization': 'token %s' % access_token
            })

            github_user, created = User.objects.get_or_create(
                application=auth.application,
                uid=user['id'],
                defaults={
                    'login': user['login'],
                    'url': user['url'],
                    'email': user['email'],
                    'access_token': access_token
                }
            )

            if not created:
                github_user.access_token = access_token
                github_user.save()

            authenticated_user = authenticate(github_user=github_user)
            if not authenticated_user is None:
                login(request, authenticated_user)



from celery import task
import os
from subprocess import call
from django.conf import settings
from github.models import Repository, User


@task()
def fetch_repo(pk, ssh_key=None):
    repository = Repository.objects.get(pk=pk)
    location = os.path.join(settings.GITHUB_ROOT,
                            'repositories',
                            repository.owner.login,
                            repository.name)

    if not os.path.exists(location):
        cmd = 'git clone %s %s' % (repository.ssh_url, location)
    else:
        os.chdir(location)
        cmd = "git pull --rebase"

    if ssh_key:
        call(['ssh-agent', 'bash', '-c', 'ssh-add %s; %s' % (ssh_key, cmd)])
    else:
        call(['bash', '-c', cmd])

    return location

@task()
def setup_private_key(pk):
    user = User.objects.get(pk=pk)
    location = os.path.join(settings.SSH_KEY_DIR,
                            'id_rsa.%s' % user.login)

    if not os.path.exists(location):
        call(['ssh-keygen', '-f %s' % location, "-N ''"])

    with open(location + ".pub") as key:
        user.add_key(user.application.name, key.read())


from django.contrib.auth.backends import ModelBackend
from utils import get_user_model

User = get_user_model()


class GithubBackend(ModelBackend):
    """
    This backend is to be used in conjunction with the ``RemoteUserMiddleware``
    found in the middleware module of this package, and is used when the server
    is handling authentication outside of Django.

    By default, the ``authenticate`` method creates ``User`` objects for
    usernames that don't already exist in the database.  Subclasses can disable
    this behavior by setting the ``create_unknown_user`` attribute to
    ``False``.
    """

    # Create a User object if not already in the database?
    create_unknown_user = True

    def authenticate(self, github_user):
        """
        The github.models.User passed as ``github_user`` is considered trusted.
        This method simply returns the ``User`` object with the given username,
        creating a new ``User`` object if ``create_unknown_user`` is ``True``.

        Returns None if ``create_unknown_user`` is ``False`` and a ``User``
        object with the given username is not found in the database.
        """
        if not github_user:
            return

        if self.create_unknown_user and not github_user.user:
            github_user.user = User.objects.create(
                username=github_user.login,
                email=github_user.email,
            )
            github_user.save()
        return github_user.user


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Key'
        db.create_table(u'github_key', (
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['github.User'])),
            ('id', self.gf('django.db.models.fields.PositiveIntegerField')(primary_key=True)),
            ('key', self.gf('django.db.models.fields.TextField')()),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=200)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=100)),
        ))
        db.send_create_signal(u'github', ['Key'])


    def backwards(self, orm):
        # Deleting model 'Key'
        db.delete_table(u'github_key')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'github.application': {
            'Meta': {'object_name': 'Application'},
            'callback_url': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'client_id': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'client_secret': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'default_scope': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'github.authentication': {
            'Meta': {'object_name': 'Authentication'},
            'application': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['github.Application']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'redirect_uri': ('django.db.models.fields.URLField', [], {'max_length': '200', 'db_index': 'True'}),
            'scope': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'})
        },
        u'github.key': {
            'Meta': {'object_name': 'Key'},
            'id': ('django.db.models.fields.PositiveIntegerField', [], {'primary_key': 'True'}),
            'key': ('django.db.models.fields.TextField', [], {}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['github.User']"})
        },
        u'github.repository': {
            'Meta': {'unique_together': "(('uid', 'owner'),)", 'object_name': 'Repository'},
            'clone_url': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {}),
            'description': ('django.db.models.fields.TextField', [], {}),
            'fork': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'forks': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'forks_count': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'git_url': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'homepage': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'html_url': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'master_branch': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'mirror_url': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'open_issues': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'repositories'", 'to': u"orm['github.User']"}),
            'private': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'pushed_at': ('django.db.models.fields.DateTimeField', [], {}),
            'size': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'ssh_url': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'svn_url': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'uid': ('django.db.models.fields.BigIntegerField', [], {}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'watchers': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'watchers_count': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'})
        },
        u'github.user': {
            'Meta': {'unique_together': "(('uid', 'application'),)", 'object_name': 'User'},
            'access_token': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'application': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['github.Application']"}),
            'avatar_url': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'login': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'uid': ('django.db.models.fields.BigIntegerField', [], {}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']", 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['github']


from django.conf.urls import patterns, include, url
from django.views.generic import TemplateView

urlpatterns = patterns(
    '',
    url(r'^github/', include('github.urls', namespace='github')),
    url(r'^test-url$', TemplateView.as_view(template_name='index.html'), name='test'),
    url(r'^$', TemplateView.as_view(template_name='index.html')),
)
